# INIparse v1.3
#      by Waster_ and djsmoke
#
# Usage:
#   readini <filename> <keyname> <itemname>
#	Reads <itemname> under <keyname> from <filename>. If it cannot find
#	file, <itemname>, or <keyname>, returns an empty string.
#
#   writeini <filename> <keyname> <itemname> <itemvalue>
#	Writes/modifies <itemname> under <keyname> in <filename>. If cannot
#	file, <itemname>, or <keyname>, creates a new one.
#	Also, deletes empty and Whitespace-Only (spaces,tabs)
#	lines in <filename>.
#
#   iniparse:removekey <filename> <keyname>
#	Deletes <keyname> and all items of it in <filename>. If cannot find
#	file returns empty string. Also, deletes empty and Whitespace-Only
#	(spaces,tabs) lines in <filename>.
#
#   iniparse:removeitem <filename> <keyname> <itemname>
#	Deletes <itemname> under <keyname> in <filename>. If cannot find
#	file returns empty string. Also, deletes empty and Whitespace-Only
#	(spaces,tabs) lines in <filename>.
#
#   iniparse:renamekey <filename> <oldkeyname> <newkeyname>
#	Renames <oldkeyname> to <newkeyname> in <filename>. If cannot find
#	file returns empty string. Also, deletes empty and Whitespace-Only
#	(spaces,tabs) lines in <filename>.
#
#   iniparse:renameitem <filename> <keyname> <olditemname> <newitemname>
#	Renames <olditemname> to <newitemname> under <keyname> in <filename>.
#	If cannot find file, returns empty string. Also, deletes empty and
#	Whitespace-Only (spaces,tabs) lines in <filename>.
#
#
# Thanks to djsmoke for "removeini" idea.
#
# Starting from v1.3, comments can only start with a "#" char, instead of
# "all meaningless lines are comments" usage.
#
# Comments from djsmoke: v1.3 implements windows ini file support to help manage
# keys/values...
# these files look something like this:
#
# [key]
# item=value
#
# Contact waster (author) at:
#	email:	waster@iname.com
#	icq  :	27216346
#
# Contact djsmoke (editor) at:
#	email:	djsmoke@linux.konect.net
#	icq  :	606314
#

proc rtrimleft {bigstr srcstr} {
  set istring1 [string length $srcstr]
  if {$srcstr == [string range $bigstr 0 [expr $istring1-1]]} {
    return [string range $bigstr $istring1 end]
  }
}

proc chrchk {args} {
  set args [string trim [string trim $args \}] \{]
  if {[string range $args 0 0] == "\[" && [string range $args end end] == "\]"} {
    return 1
  } else {return 0}
  return 0
}

proc readini {file key item} {
  set keyfound 0 ; set itemfound 0
  set itemlength [string length $item]
  set item "$item="

  set fileId [open $file r]

  while ![eof $fileId] {
    if {[eof $fileId]} {break}
    set input [gets $fileId]
    if {[chrchk $input]} {
      if {$input == "\[$key\]"} {
        # looks like we found our key..
        set keyfound 1 ; break
      } else {set keyfound 0}
    }
  }

  if {$keyfound == 1} {
    while {![eof $fileId]} {
      set input "[gets $fileId]"

      if {![chrchk $input]} {
        if {[string range $input 0 $itemlength]==$item} {
          set fValue [rtrimleft $input $item]
          set itemfound 1
          break
        }
      } elseif {[chrchk $input]} {
        # oops, we hit the next key.. looks like that item isnt here.
        seek $fileId 0 end
      }
      if {[eof $fileId]} {break}
    }
  } elseif {$keyfound == 0} {
    catch {close $fileId}
    return "error: no key exists ($key)"
  }
  catch {close $fileId}
  if {$itemfound == 1} {return "$fValue"
  } else {return "error: no item exists ($item)"}
}

proc writeini {file key item value} {
  set fileline ""
  set keyfound 0
  set itemfound 0
  set itemlength [string length $item]
  set item "$item="

  if {[catch {set fname [open $file r+]}] != 0} {set fname [open $file w+]}

  while {![eof $fname]} {
    set input [gets $fname]
    if {[string match {\[*\]} $input] || \
	[string match {*=*} $input] || \
	[string match {#*} $input]} {lappend fileline $input}
  }

  close $fname
  set fname [open $file w+]	;	# We cause the file to be emptied

  foreach line $fileline {
    if {(!$keyfound)} {          ;# We will search for the key.
	if {$line=="\[$key\]"} {
		puts $fname "\n$line"
		set keyfound 1
		continue
	}
    }
    if {($keyfound) && (!$itemfound) && [chrchk $line]} {  ;# We hit the next key!
	puts $fname "$item$value"
	puts $fname "\n$line"
	set itemfound 1
	continue
    }
    if {($keyfound) && (!$itemfound)} {  ;# We will search for the item
	if {[string range $line 0 $itemlength]==$item} {
		puts $fname "$item$value"
		set itemfound 1
		continue
	}
    }
    # Because it doesn't match anything above, we print the line to file
    if {[chrchk $line]} {puts $fname "\n$line"} \
	else {puts $fname $line}
  }

  if {(!$keyfound) && (!$itemfound)} {
	puts $fname "\n\[$key\]"
	puts $fname "$item$value"
  } elseif {$keyfound && (!$itemfound)} {
	puts $fname "$item$value"
  }
  close $fname
  return 1
}

proc iniparse:removeitem {file key item} {
  set fileline ""
  set keyfound 0
  set itemfound 0
  set itemlength [string length $item]
  set item "$item="

  # return if cannot open file (usually when it doesn't exist)
  if {[catch {set fname [open $file r+]}] != 0} {return}

  while {![eof $fname]} {
    set input [gets $fname]
    if {[string trim $input] != ""} {lappend fileline $input}
  }

  close $fname
  set fname [open $file w+]	;	# We cause the file to be emptied

  foreach line $fileline {
    if {(!$keyfound)} {          ;# We will search for the key.
	if {$line=="\[$key\]"} {
		puts $fname "\n$line"
		set keyfound 1
		continue
	} else {puts $fname $line; continue}
    }
    if {($keyfound) && (!$itemfound) && [chrchk $line]} {  ;# We hit the next key!
	puts $fname "\n$line"
	set itemfound 1
	continue
    }
    if {($keyfound) && (!$itemfound)} {  ;# We will search for the item
	if {[string range $line 0 $itemlength]==$item} {
		set itemfound 1
		continue        ;# We don't print the line to file
	} else {puts $fname $line; continue}
    }
    # Because it doesn't match anything above, we print the line to file
    if {[chrchk $line]} {puts $fname "\n$line"} \
	else {puts $fname $line}
  }
  close $fname
}

proc iniparse:removekey {file key} {
  set fileline ""
  set keyfound 0
  set keyfinished 0

  # return if cannot open file (usually when it doesn't exist)
  if {[catch {set fname [open $file r+]}] != 0} {return}

  while {![eof $fname]} {
    set input [gets $fname]
    if {[string trim $input] != ""} {lappend fileline $input}
  }

  close $fname
  set fname [open $file w+]	;	# We cause the file to be emptied

  foreach line $fileline {
    if {(!$keyfound)} {          ;# We will search for the key.
	if {$line=="\[$key\]"} {
		set keyfound 1
		continue
	}
    }
    if {($keyfound) && (!$keyfinished) && (![chrchk $line])} {  ;# We are waiting for the next key
	continue
    }
    if {($keyfound) && (!$keyfinished) && ([chrchk $line])} {  ;# We are hit the next key!
	puts $fname "\n$line"
	set keyfinished 1
	continue
    }
    # Because it doesn't match anything above, we print the line to file
    if {[chrchk $line]} {puts $fname "\n$line"} \
	else {puts $fname $line}
  }
  close $fname
}

proc iniparse:renameitem {file key olditem newitem} {
  writeini $file $key $newitem [readini $file $key $olditem]
  iniparse:removeitem $file $key $olditem
}

proc iniparse:renamekey {file oldkey newkey} {
  set fileline ""

  # return if cannot open file (usually when it doesn't exist)
  if {[catch {set fname [open $file r+]}] != 0} {return}

  while {![eof $fname]} {
    set input [gets $fname]
    if {[string trim $input] != ""} {lappend fileline $input}
  }

  close $fname
  set fname [open $file w+]	;	# We cause the file to be emptied

  foreach line $fileline {
	if {$line=="\[$oldkey\]"} {
		puts $fname "\n\[$newkey\]"
		continue
	} else {
		if {[chrchk $line]} {puts $fname "\n$line"} \
		else {puts $fname $line}
	}
  }
  close $fname
}
